@@ -3,14 +3,14 @@ from office365.actions.search_query import SearchQuery
 from office365.actions.upload_content_query import UploadContentQuery
 from office365.directory.directory import Directory
 from office365.directory.directoryObjectCollection import DirectoryObjectCollection
-from office365.directory.groupCollection import GroupCollection
+from office365.directory.group_collection import GroupCollection
 from office365.directory.groupSettingTemplateCollection import GroupSettingTemplateCollection
 from office365.directory.user import User
 from office365.directory.userCollection import UserCollection
 from office365.onedrive.driveCollection import DriveCollection
 from office365.onedrive.sharedDriveItemCollection import SharedDriveItemCollection
 from office365.onedrive.siteCollection import SiteCollection
-from office365.outlookservices.contact_collection import ContactCollection
+from office365.outlook.contact_collection import ContactCollection
 from office365.runtime.auth.token_response import TokenResponse
 from office365.runtime.client_runtime_context import ClientRuntimeContext
 from office365.runtime.http.http_method import HttpMethod
@@ -20,7 +20,7 @@ from office365.runtime.odata.v4_json_format import V4JsonFormat
 from office365.runtime.queries.delete_entity_query import DeleteEntityQuery
 from office365.runtime.queries.update_entity_query import UpdateEntityQuery
 from office365.runtime.resource_path import ResourcePath
-from office365.teams.teamCollection import TeamCollection
+from office365.teams.team_collection import TeamCollection
 
 
 class GraphClient(ClientRuntimeContext):
