@@ -11,12 +11,14 @@ class FieldCreationInformation(ClientValue):
         Represents metadata about fields creation.
 
         :type title: str
+        :type field_type_kind: int
+        :type description: str or None
         """
         super(FieldCreationInformation, self).__init__()
         self.Title = title
         self.FieldTypeKind = field_type_kind
         self.Description = description
-        self.Choices = field_type_kind == FieldType.MultiChoice and ClientValueCollection(str) or None
+        self.Choices = ClientValueCollection(str) if field_type_kind == FieldType.MultiChoice else None
         self.LookupListId = None
         self.LookupFieldName = None
         self.LookupWebId = None
