@@ -2,5 +2,6 @@ class ExternalSharingSiteOption:
     def __init__(self):
         pass
 
-    Edit = "role:1073741827"
-    View = "role:1073741826"
+    View = "View"
+    Edit = "Edit"
+    Owner = "Owner"
